var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { TokenUtil } from "./TokenUtilities";
import { ApiAccessBuilder } from "./RequestWrapper";
import { RBAuthInitialToken } from "../roles-based-auth/context";
import { RBAuthErrors } from "../rb-auth-errors";
class UserReloader {
    constructor(setReloading, setUser) {
        this.setReloading = setReloading;
        this.setUser = setUser;
    }
}
const dummyFunction = (arg) => new Promise((resolve) => resolve(arg));
export class BaseAuthApiWrapper extends UserReloader {
    constructor(setReloading, setUser, processes, refreshTokenEndLifeCallback, appApis) {
        super(setReloading, setUser);
        this.defaultLogin = dummyFunction;
        this.defaultLogout = dummyFunction;
        this.defaultSignupLogic = dummyFunction;
        this.defaultSilentLogic = dummyFunction;
        this.defaultHandleLogic = dummyFunction;
        this.defaultRefreshLogic = dummyFunction;
        this.loginLogic = this.defaultLogin;
        this.logoutLogic = this.defaultLogout;
        this.signupLogic = this.defaultSignupLogic;
        this.silentLogic = this.defaultSilentLogic;
        this.handleLogic = this.defaultHandleLogic;
        this.refreshLogic = this.defaultRefreshLogic;
        this.errorCallback = () => null;
        this.appApis = {};
        this.login = (...args) => __awaiter(this, void 0, void 0, function* () { return this.runLogic(this.loginLogic)(...args); });
        this.handle = (...args) => __awaiter(this, void 0, void 0, function* () { return this.runLogic(this.handleLogic)(...args); });
        this.silent = (...args) => __awaiter(this, void 0, void 0, function* () { return this.runLogic(this.silentLogic)(...args); });
        // simple signup
        this.signup = (...args) => this.signupLogic(...args);
        // custom logic
        this.refresh = (f) => __awaiter(this, void 0, void 0, function* () { return this.runRefresh(this.refreshLogic)(f); });
        this.logout = (...args) => __awaiter(this, void 0, void 0, function* () {
            return this.wrap(() => __awaiter(this, void 0, void 0, function* () {
                const res = yield this.logoutLogic(...args);
                yield this.authenticate();
                return res;
            }));
        });
        this.runLogic = (logic) => (...args) => __awaiter(this, void 0, void 0, function* () {
            return this.wrap(() => __awaiter(this, void 0, void 0, function* () {
                const res = yield logic(...args);
                // console.log('res', res);
                yield this.authenticate(res);
                return res;
            }));
        });
        this.runRefresh = (logic) => (flag) => __awaiter(this, void 0, void 0, function* () {
            return this.wrap(() => __awaiter(this, void 0, void 0, function* () {
                const res = yield logic();
                if (res.refreshToken)
                    yield TokenUtil.setTokens(res);
                return res;
            }), flag);
        });
        // this means the access token got krazy!
        this.accessTokenError = RBAuthErrors.UNAUTHORIZED;
        this.refreshTokenError = RBAuthErrors.INVALID_GRANT;
        this.apiWrap = (logic) => (onSuccess, onError) => {
            // console.log('inside apiWrap');
            return new ApiAccessBuilder(logic)
                .withSuccess(onSuccess)
                .withFailure(onError)
                .withAccessTokenError(this.accessTokenError)
                .withRefreshTokenError(this.refreshTokenError)
                .build(this.refresh);
        };
        this.wrap = (logic, flag = true) => __awaiter(this, void 0, void 0, function* () {
            if (flag)
                this.setReloading(true);
            try {
                return yield logic();
            }
            catch (e) {
                // console.log('wrap catch: ', e.message);
                if (e.message === "Failed to fetch")
                    this.errorCallback(RBAuthErrors.FAILLED_TO_FETCH, e);
                // others
                else if (e.message === RBAuthErrors.UNAUTHORIZED)
                    this.errorCallback(e);
                else if (e.message === RBAuthErrors.REFRESH_TOKEN_NOT_PRESENT) {
                    // swallow silently
                }
                // all others
                else if (e.message !== RBAuthErrors.REFRESH_TOKEN_NOT_PRESENT)
                    this.errorCallback(RBAuthErrors.UNKNOWN, e);
                throw e;
            }
            finally {
                if (flag)
                    this.setReloading(false);
            }
            // try {
            //   return logic();
            // } catch (error) {
            //   if (error.message === AuthErrors.REFRESH_TOKEN_NOT_PRESENT) {
            //     // swallow
            //   }
            //   //  else if (error.message === AuthErrors.REFRESH_TOKEN_REVOKED) {
            //   //   this.errorCallback(AuthErrors.REFRESH_TOKEN_REVOKED);
            //   // } else if (
            //   //   error.mesage === AuthErrors.UNAUTHORIZED ||
            //   //   error.mesage === 'JSON Parse error: Unexpected identifier "Unauthorized"'
            //   // ) {
            //   //   this.errorCallback(AuthErrors.UNAUTHORIZED);
            //   // } else if (error.message === AuthErrors.TOO_MANY_REQUESTS) {
            //   //   this.errorCallback(AuthErrors.TOO_MANY_REQUESTS);
            //   // }
            //   console.log('wrap catch: ', error, error.message);
            //   // throw error;
            //   return null;
            // } finally {
            //   if (flag) this.setReloading(false);
            // }
        });
        this.authenticate = (res = null) => __awaiter(this, void 0, void 0, function* () {
            if (res && res.user && res.tokens) {
                this.setUser(res.user);
                yield TokenUtil.setTokens(res.tokens);
            }
            else {
                this.setUser(null);
                yield TokenUtil.setTokens(RBAuthInitialToken);
            }
        });
        if (refreshTokenEndLifeCallback)
            this.errorCallback = refreshTokenEndLifeCallback;
        if (appApis)
            this.appApis = this.embedWrapperLogicIntoApis(appApis);
        if (!processes)
            return;
        if (processes.login)
            this.loginLogic = processes.login;
        if (processes.logout)
            this.logoutLogic = processes.logout;
        if (processes.signup)
            this.signupLogic = processes.signup;
        if (processes.handle)
            this.handleLogic = processes.handle;
        if (processes.silent)
            this.silentLogic = processes.silent;
        if (processes.refresh)
            this.refreshLogic = processes.refresh;
    }
    get apis() {
        return this.appApis;
    }
    /**
     * private Helpers
     */
    embedWrapperLogicIntoApis(apis) {
        const result = {};
        Object.keys(apis).forEach((item) => {
            if (item.startsWith("logic") &&
                apis[item] &&
                typeof apis[item] === "function")
                result[item] = this.apiWrap(apis[item]);
            else if (apis[item] !== null && typeof apis[item] === "object")
                result[item] = this.embedWrapperLogicIntoApis(apis[item]);
            else
                result[item] = apis[item];
        });
        return result;
    }
}
//# sourceMappingURL=BaseAuthApiWrapper.js.map