var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { DEFAULT_TIMEOUT, fetchWithTimeout } from "../utils/FetchWithTimeout";
export var HTTPMethod;
(function (HTTPMethod) {
    HTTPMethod["POST"] = "POST";
    HTTPMethod["GET"] = "GET";
    HTTPMethod["PATCH"] = "PATCH";
    HTTPMethod["PUT"] = "PUT";
    HTTPMethod["DELETE"] = "DELETE";
    HTTPMethod["HEAD"] = "HEAD";
    HTTPMethod["CONNECT"] = "CONNECT";
    HTTPMethod["TRACE"] = "TRACE";
})(HTTPMethod || (HTTPMethod = {}));
export class RequestBuilder {
    constructor(route, debug = false) {
        this.debug = debug;
        this.route = "";
        this.body = null;
        this.headers = new Headers();
        this.method = HTTPMethod.GET;
        this.mode = null;
        this.errorHandling = null;
        this.timeout = DEFAULT_TIMEOUT;
        this.route = route;
        return this;
    }
    withAuth0Body(body = {}) {
        this.body = body;
        return this;
    }
    withErrorHandling(callback) {
        this.errorHandling = callback;
        return this;
    }
    withBody(body = {}) {
        this.body = body;
        return this;
    }
    withHeaders(headers) {
        this.headers = headers;
        return this;
    }
    withMethod(method) {
        this.method = method;
        return this;
    }
    withMode(mode) {
        this.mode = mode;
        return this;
    }
    withTimeout(timeout) {
        this.timeout = timeout;
        return this;
    }
    request() {
        if (this.debug) {
            const debugHeaders = {};
            this.headers.forEach((value, key) => (debugHeaders[key] = value));
            // eslint-disable-next-line no-console
            console.log("will request: ", this.route, this.method, JSON.stringify(debugHeaders), JSON.stringify(this.body));
        }
        const opts = {
            method: this.method,
            headers: this.headers,
        };
        if (this.mode)
            opts.mode = this.mode;
        if (this.method !== HTTPMethod.GET &&
            this.method !== HTTPMethod.HEAD &&
            this.body)
            opts.body = JSON.stringify(this.body);
        return fetchWithTimeout(this.route, opts, this.timeout);
    }
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            let result;
            const res = yield this.request();
            const contentType = res.headers.get("content-type");
            const success = res.ok;
            if (contentType && contentType.indexOf("application/json") !== -1)
                result = yield res.json();
            else
                result = (yield res.text());
            if (!success) {
                if (this.errorHandling)
                    this.errorHandling(result);
                throw Error(result);
            }
            return result;
        });
    }
}
//# sourceMappingURL=RequestBuilder.js.map