var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { RBAuthErrors } from "../rb-auth-errors";
const RECUR_LEVEL = 1;
class RequestWrapper {
    /**
     * RequestWrapper::handle()
     *
     * @param request Actual request to be performed
     * @param onSuccess Receives res: <T>(res: T) => void as success logic
     * @param onFailure Receives error: <T>(error: T) => void as failure logic
     */
    static handle(request, onSuccess, onFailure, recursion = RECUR_LEVEL, accessTokenError = RBAuthErrors.UNAUTHORIZED, refresTokenError = RBAuthErrors.INVALID_GRANT) {
        /**
         * onPress logic
         * @param refresh Logic for refreshing tokens
         */
        return (refreshLogic) => () => __awaiter(this, void 0, void 0, function* () {
            return RequestWrapper.recursion(refreshLogic, request, onSuccess, onFailure, recursion, accessTokenError, refresTokenError);
        });
    }
}
RequestWrapper.recursion = (refresh, req, onSuccess, onFailure, recursion = RECUR_LEVEL, accessTokenError = RBAuthErrors.UNAUTHORIZED, refresTokenError = RBAuthErrors.INVALID_GRANT) => __awaiter(void 0, void 0, void 0, function* () {
    // console.log('RequestWrapper::recursion');
    try {
        // console.log('RequestWrapper::recursion will request');
        yield req();
        // console.log('RequestWrapper::recursion has request');
    }
    catch (e) {
        if (e.message === accessTokenError && recursion >= 1) {
            try {
                // refresh
                yield refresh();
                // then
                return RequestWrapper.recursion(refresh, req, onSuccess, onFailure, recursion - 1, accessTokenError, refresTokenError);
            }
            catch (error) {
                onFailure(RBAuthErrors.REFRESH_TOKEN_REVOKED);
                throw new Error(RBAuthErrors.REFRESH_TOKEN_REVOKED);
            }
        }
        else {
            onFailure(e);
        }
        throw e;
    }
});
export class ApiAccessBuilder {
    constructor(logic) {
        this.success = () => null;
        this.failure = () => null;
        this.recursions = RECUR_LEVEL;
        this.accessTokenError = RBAuthErrors.INVALID_GRANT;
        this.refreshTokenError = RBAuthErrors.REFRESH_TOKEN_REVOKED;
        this.logic = logic;
    }
    withAccessTokenError(accessTokenError) {
        this.accessTokenError = accessTokenError;
        return this;
    }
    withRefreshTokenError(refreshTokenError) {
        this.refreshTokenError = refreshTokenError;
        return this;
    }
    withSuccess(success) {
        this.success = success;
        return this;
    }
    withFailure(failure) {
        this.failure = failure;
        return this;
    }
    withCustomRecursions(recursions) {
        this.recursions = recursions;
        return this;
    }
    build(refresh) {
        return RequestWrapper.handle(this.logic, this.success, this.failure, this.recursions, this.accessTokenError, this.refreshTokenError)(refresh);
    }
}
//# sourceMappingURL=RequestWrapper.js.map