import { UnknownAuthProcess, SetterType, AuthApiInterface, RBAuthTokensType, RBAuthUserModelWithRole, KnownAuthProcess, RBAuthBaseRoles } from "..";
import { RBAuthErrors } from "../rb-auth-errors";
declare type AuthProcessResponse = KnownAuthProcess<{
    tokens: RBAuthTokensType;
    user: RBAuthUserModelWithRole<RBAuthBaseRoles>;
}>;
declare type RefreshProcessResponse = KnownAuthProcess<RBAuthTokensType>;
declare type ProcessesType<TLoging, LogOutType, SignUpType, SilentAuthType, HandleAuthType, RefreshTokens> = {
    login?: TLoging;
    logout?: LogOutType;
    signup?: SignUpType;
    silent?: SilentAuthType;
    handle?: HandleAuthType;
    refresh?: RefreshTokens;
};
declare abstract class UserReloader {
    setReloading: SetterType;
    setUser: SetterType;
    constructor(setReloading: SetterType, setUser: SetterType);
}
declare type RefreshTokenEndLifeCallbackType = (rbAuthErros: RBAuthErrors, error?: Error) => void;
export declare class BaseAuthApiWrapper<LoginType extends KnownAuthProcess<{
    tokens: RBAuthTokensType;
    user: RBAuthUserModelWithRole<RBAuthBaseRoles>;
}>, LogOutType extends UnknownAuthProcess, SignUpType extends KnownAuthProcess<unknown>, SilentType extends KnownAuthProcess<{
    tokens: RBAuthTokensType;
    user: RBAuthUserModelWithRole<RBAuthBaseRoles>;
}>, HandleType extends KnownAuthProcess<{
    tokens: RBAuthTokensType;
    user: RBAuthUserModelWithRole<RBAuthBaseRoles>;
}>, RefreshType extends KnownAuthProcess<RBAuthTokensType>, AppApis extends Record<string, unknown>> extends UserReloader implements AuthApiInterface {
    defaultLogin: LoginType;
    defaultLogout: LogOutType;
    defaultSignupLogic: SignUpType;
    defaultSilentLogic: SilentType;
    defaultHandleLogic: HandleType;
    defaultRefreshLogic: RefreshType;
    private loginLogic;
    private logoutLogic;
    private signupLogic;
    private silentLogic;
    private handleLogic;
    private refreshLogic;
    private errorCallback;
    private appApis;
    constructor(setReloading: SetterType, setUser: SetterType, processes: ProcessesType<LoginType, LogOutType, SignUpType, SilentType, HandleType, RefreshType>, refreshTokenEndLifeCallback?: RefreshTokenEndLifeCallbackType, appApis?: AppApis);
    get apis(): Record<string, unknown>;
    login: AuthProcessResponse;
    handle: AuthProcessResponse;
    silent: AuthProcessResponse;
    signup: UnknownAuthProcess;
    refresh: RefreshProcessResponse;
    logout: UnknownAuthProcess;
    /**
     * private Helpers
     */
    private embedWrapperLogicIntoApis;
    private runLogic;
    private runRefresh;
    private accessTokenError;
    private refreshTokenError;
    private apiWrap;
    private wrap;
    private authenticate;
}
export {};
